<?php
namespace WSCOMUN\PORTAFIRMAS;
use WSCMIME\WSCMimeParser;
use WSCOMUN\WSComunFClient;
use ArrayObject;
use Exception;
use SoapVar;
/**
 * Librera para que aplicaciones PHP/GVHidra puedan acceder al Web Service de Portafirmas
 *
 * @package		WSClientPortafirmas
 * @author 		David Pascual <pascual_dav@externosgva.es>
 * @version		Release: @package_version@
 * @copyright 	Copyright (C) 2015 Generalitat Valenciana.
 * @license 	http://opensource.org/licenses/gpl-2.0.php GNU Public License v2.0
 * @link 		https://gvhidra.gva.es/redmine/projects/wscomun
 */
class WSClientPortafirmas extends WSComunFClient
{

	/**
	 *
	 * Crea el tag de autenticacin comun a todas las peticiones del portafirmas, y lo devuelve dentro de un ArrayObject
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $fieldName Tag en el que se encapsularan los datos
	 * @param string $ns NS del tag
	 * @return ArrayObject	ArrayObject del nivel del tag, al que se deben aadir los tags restantes
	 * @access 	private
	 */
	private function pf_doAuth($username, $password, $fieldName, $ns) {
		$oAux = new authentication();
		$oAux->setUserName($username);
		$oAux->setPassword($password);
		$authWS = $oAux->toSoap($fieldName, $ns);

		$mainWS = new ArrayObject();
		$mainWS->append(new SoapVar($authWS, SOAP_ENC_OBJECT,null,null, $fieldName, $ns));
		return $mainWS;
	}

	/**
	 *
	 * Permite recuperar el contenido de un documento (binario y en B64) a partir del id del mismo
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $documentId Identificador del documento
	 * @return	array 	Array asociativo con la respuesta del servicio [documentBinary|documentB64]
	 * @access 	public
	 */
	public function pf_downloadDocument($username, $password, $documentId)
	{
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($documentId, XSD_STRING, null, null,'documentId', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'downloadDocument', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->downloadDocument($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			switch ($mensaje)
			{
				case 'looks like we got no xml document' :
					// Obtiene la respuesta
					$response = $clienteWS->__getLastResponse();
					// Realiza la interpretacion/sustitucion estandar de MTOM
					$vRetorno = $this->tratarMTOMEstandar($response);

					$vRetorno['documentB64'] = $vRetorno['documentBinary'];
					$vRetorno['documentBinary'] = base64_decode($vRetorno['documentBinary']);
					return ($vRetorno);
				break;
				default:
					$this->tratarExcepcionEstandar($e, $clienteWS);
				break;
			}
		}
	}//pf_downloadDocument


	/**
	 *
	 * Permite recuperar la firma de un documento (array asociativo) a partir del id del documento
	 *
	 * @param 	string 	$username Nombre de usuario
	 * @param 	string 	$password Contrasea del usuario
	 * @param 	string 	$documentId Identificador del documento
	 * @return	array 	Array asociativo [signature] [[identifier],[sign],[signFormat],[csv],[csvAnulado],[csvNormativa],[content]]
	 * @access 	public
	 */
	public function pf_downloadSign($username, $password, $documentId) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($documentId, XSD_STRING, null, null,'documentId', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'downloadSign', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->downloadSign($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			switch ($mensaje)
			{
				case 'looks like we got no xml document' :
					// Obtiene la respuesta
					$response = $clienteWS->__getLastResponse();
					// Realiza la interpretacion/sustitucion estandar de MTOM
					$vRetorno = $this->tratarMTOMEstandar($response);
					if (isset($vRetorno['signature']))
					{
						$vRetorno['signature']['contentB64'] = $vRetorno['signature']['content'];
						$vRetorno['signature']['content'] = base64_decode($vRetorno['signature']['content']);
					}
					return ($vRetorno);
				break;
				default:
					$this->tratarExcepcionEstandar($e, $clienteWS);
				break;
			}
		}
	}//pf_downloadSign


	/**
	 *
	 * Permite recuperar los tipos de documento disponibles
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Valor de bsqueda o null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryDocumentTypes($username, $password, $query) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($query, XSD_STRING, null, null,'query', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryDocumentTypes', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryDocumentTypes($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryDocumentTypesResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryDocumentTypes


	/**
	 *
	 * Permite recuperar los cargos disponibles
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Valor de bsqueda o null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryJobs($username, $password, $query) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($query, XSD_STRING, null, null,'query', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryJobs', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryJobs($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryJobsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryJobs


	/**
	 *
	 * Permite  recuperar  una  peticin  existente  en  el  servidor
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion.
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryRequest($username, $password, $requestId) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryRequest', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryRequest($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryRequestResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryRequest


	/**
	 *
	 * Permite recuperar los estados disponibles
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Valor de bsqueda o null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryStates($username, $password, $query) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($query, XSD_STRING, null, null,'query', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryStates', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryStates($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryStatesResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryStates


	/**
	 *
	 * Permite recuperar los usuarios disponibles.
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Valor de bsqueda o null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryUsers($username, $password, $query) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($query, XSD_STRING, null, null, 'query', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT, null, null, 'queryUsers', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryUsers($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryUsersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryUsers


	/**
	 *
	 * Permite recuperar los niveles de importancia disponibles
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Valor de bsqueda o null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryImportanceLevels($username, $password, $query) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($query, XSD_STRING, null, null, 'query', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryImportanceLevels', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryImportanceLevels($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryImportanceLevelsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryImportanceLevels


	/**
	 *
	 * Permite recuperar una lista de usuarios vlidos con algo ms de informacin a partir de una consulta
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $queryUser Filtro de usuarios / Null
	 * @param string $querySeat Filtro de sedes / Null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryEnhancedUsers($username, $password, $queryUser, $querySeat) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($queryUser, XSD_STRING, null, null,'queryUser', self::$NAMESPACE_PF));
			$mainWS->append(new SoapVar($querySeat, XSD_STRING, null, null,'querySeat', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryEnhancedUsers', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryEnhancedUsers($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryEnhancedUsersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryEnhancedUsers


	/**
	 *
	 * Permite recuperar una lista de cargos vlidos con algo ms de informacin a partir de una consulta
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $queryJob Filtro de cargo / Null
	 * @param string $querySeat Filtro de sedes / Null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryEnhancedJobs($username, $password, $queryJob, $querySeat) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($queryJob, XSD_STRING, null, null,'queryJob', self::$NAMESPACE_PF));
			$mainWS->append(new SoapVar($querySeat, XSD_STRING, null, null,'querySeat', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryEnhancedJobs', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryEnhancedJobs($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryEnhancedJobsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryEnhancedJobs


	/**
	 *
	 * Recupera la lista de usuarios asociados a un cargo. Un cargo slo puede tener asociado a un usuario en un determinado periodo de tiempo
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $jobIdentifier Identificador del cargo
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryEnhancedUserJobAssociatedToJob($username, $password, $jobIdentifier) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($jobIdentifier, XSD_STRING, null, null,'jobIdentifier', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryEnhancedUserJobAssociatedToJob', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryEnhancedUserJobAssociatedToJob($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryEnhancedUserJobAssociatedToJobResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryEnhancedUserJobAssociatedToJob


	/**
	 *
	 * Recupera la lista de cargos asociados a un usuario
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $userIdentifier Identificador del usuario
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_queryEnhancedUserJobAssociatedToUser($username, $password, $userIdentifier) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($userIdentifier, XSD_STRING, null, null, 'userIdentifier', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'queryEnhancedUserJobAssociatedToUser', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->queryEnhancedUserJobAssociatedToUser($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'queryEnhancedUserJobAssociatedToUserResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_queryEnhancedUserJobAssociatedToJob


	/**
	 *
	 * Recupera las sedes que cumplen una determinada condicin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Valor de bsqueda o null
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_querySeats($username, $password, $query) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($query, XSD_STRING, null, null,'query', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'querySeats', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->querySeats($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'querySeatsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_querySeats


	/**
	 *
	 * Permite  recuperar  la  informacin  del  Cdigo  Seguro  de  Validacin  (CSV) asociado al documento as como la referencia del mismo en el sistema GDE
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $query Identificador del documento
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_querySign($username, $password, $documentId) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF);
			$mainWS->append(new SoapVar($documentId, XSD_STRING, null, null,'documentId', self::$NAMESPACE_PF));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'querySign', self::$NAMESPACE_PF);
			$respuesta = $clienteWS->querySign($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'querySignResponse');
				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_querySign


	/**
	 *
	 * Sirve  para  crear  una  peticin  en  el  portafirmas
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param request $request Datos de la peticion
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_createRequest($username, $password, $request) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append($request->toSoap('request', self::$NAMESPACE_PF_MOD));

			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'createRequest', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->createRequest($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'createRequestResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_createRequest


	/**
	 *
	 * Permite quitar un documento de una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $documentId Identificador del documento
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_deleteDocument($username, $password, $documentId) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($documentId, XSD_STRING, null, null,'documentId', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'deleteDocument', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->deleteDocument($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'deleteDocumentResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_deleteDocument


	/**
	 *
	 * Permite eliminar una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_deleteRequest($username, $password, $requestId) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'deleteRequest', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->deleteRequest($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'deleteRequestResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_deleteRequest


	/**
	 *
	 * Permite eliminar firmantes de una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion
	 * @param signerList $signerList Lista de firmantes
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_deleteSigners($username, $password, $requestId, $signerList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF_MOD));
			$mainWS->append($signerList->toSoap('signerList', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'deleteSigners', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->deleteSigners($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'deleteSignersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_deleteSigners


	/**
	 *
	 * Permite aadir un documento a una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion
	 * @param document $document Declaracin del documento
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_insertDocument($username, $password, $requestId, $document) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF_MOD));
			$mainWS->append($document->toSoap('document', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'insertDocument', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->insertDocument($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'insertDocumentResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_insertDocument


	/**
	 *
	 * Permite aadir una nueva linea de firma a una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion
	 * @param int $signLine Linea en la que se inserta la firma
	 * @param signerList $signerList Lista de cargos y usuarios que se quieren quitar de la linea
	 * @param string $signLineType Indica si debe ser firmada o se debe dar el visto bueno [FIRMA|VISTOBUENO]
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_insertSigners($username, $password, $requestId, $signLine, $signerList, $signLineType) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF_MOD));
			$mainWS->append(new SoapVar($signLine, XSD_INT,null,null, 'signLine', self::$NAMESPACE_PF_MOD));
			$mainWS->append($signerList->toSoap('signerList', self::$NAMESPACE_PF_MOD));
			$mainWS->append(new SoapVar($signLineType, XSD_STRING, null, null, 'signLineType', self::$NAMESPACE_PF_MOD));
			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'insertSigners', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->insertSigners($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'insertSignersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_insertSigners


	/**
	 *
	 * Permite enviar una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_sendRequest($username, $password, $requestId) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'sendRequest', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->sendRequest($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'sendRequestResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_sendRequest


	/**
	 *
	 * Permite actualizar la informacin una peticin que ha sido creada, pero que todava no ha sido enviada
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param request $request Elemento complejo que representa una peticin
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_updateRequest($username, $password, $request) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append($request->toSoap('request', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'updateRequest', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->updateRequest($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'updateRequestResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_updateRequest


	/**
	 *
	 * Permite retirar una peticin enviada que no se encuentra en un estado final
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param string $requestId Identificador de la peticion
	 * @param string $removingMessage Mensaje de usuario con el motivo del retiro de la peticin
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_mod_removeRequest($username, $password, $requestId, $removingMessage) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_MOD, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_MOD);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_MOD);
			$mainWS->append(new SoapVar($requestId, XSD_STRING, null, null,'requestId', self::$NAMESPACE_PF_MOD));
			$mainWS->append(new SoapVar($removingMessage, XSD_STRING, null, null,'removingMessage', self::$NAMESPACE_PF_MOD));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'removeRequest', self::$NAMESPACE_PF_MOD);
			$respuesta = $clienteWS->removeRequest($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'removeRequestResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_mod_removeRequest


	/**
	 *
	 * Crea nuevos usuarios en la aplicacin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param enhancedUserList $enhancedUserList Lista de usuarios a insertar
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_insertEnhancedUsers($username, $password, $enhancedUserList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append($enhancedUserList->toSoap('enhancedUserList', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'insertEnhancedUsers', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->insertEnhancedUsers($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'insertEnhancedUsersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_insertEnhancedUsers


	/**
	 *
	 * Modifica  usuarios  en  la  aplicacin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param enhancedUserList $enhancedUserList Lista de usuarios a modificar
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_updateEnhancedUsers($username, $password, $enhancedUserList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append($enhancedUserList->toSoap('enhancedUserList', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'updateEnhancedUsers', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->updateEnhancedUsers($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'updateEnhancedUsersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_updateEnhancedUsers


	/**
	 *
	 * Elimina  usuarios de la aplicacin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param object userIdentifierList $userIdentifierList Lista de identificadores de usuario a eliminar
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_deleteUsers($username, $password, $userIdentifierList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append($userIdentifierList->toSoap('userIdentifierList', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'deleteUsers', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->deleteUsers($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'deleteUsersResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_deleteUsers


	/**
	 *
	 * Crea  nuevos cargos  en la  aplicacin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param enhancedJobList $enhancedJobList Lista de cargos a insertar
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_insertEnhancedJobs($username, $password, $enhancedJobList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append($enhancedJobList->toSoap('enhancedJobList', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'insertEnhancedJobs', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->insertEnhancedJobs($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'insertEnhancedJobsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_insertEnhancedJobs


	/**
	 *
	 * Modifica cargos en la aplicacin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param enhancedJobList $enhancedJobList Lista de cargos a modificar
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_updateEnhancedJobs($username, $password, $enhancedJobList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append($enhancedJobList->toSoap('enhancedJobList', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'updateEnhancedJobs', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->updateEnhancedJobs($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'updateEnhancedJobsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_updateEnhancedJobs


	/**
	 *
	 * Elimina  cargos  de  la  aplicacin
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param object jobIdentifierList $jobIdentifierList Lista de identificadores de cargos a eliminar
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_deleteJobs($username, $password, $jobIdentifierList) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append($jobIdentifierList->toSoap('jobIdentifierList', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'deleteJobs', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->deleteJobs($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'deleteJobsResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_deleteJobs


	/**
	 *
	 * Asigna un cargo a un usuario para un determinado periodo de tiempo
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param object jobIdentifier $jobIdentifier Identificador del cargo que se quiere asignar al usuario
	 * @param object userIdentifier $userIdentifier Identificador del usuario al que se le quiere asignar el cargo
	 * @param \DateTime $fstart Fecha de inicio de la asociacin usuario-cargo
	 * @param \DateTime $fend Fecha  de  finalizacin  de  la  asociacin
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_assignJobToUser($username, $password, $jobIdentifier, $userIdentifier, $fstart, $fend) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append(new SoapVar($jobIdentifier, XSD_STRING, null, null,'jobIdentifier', self::$NAMESPACE_PF_ADM));
			$mainWS->append(new SoapVar($userIdentifier, XSD_STRING, null, null,'userIdentifier', self::$NAMESPACE_PF_ADM));
			$mainWS->append(new SoapVar($fstart, XSD_DATE, null, null,'fstart', self::$NAMESPACE_PF_ADM));
			$mainWS->append(new SoapVar($fend, XSD_DATE, null, null,'fend', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'assignJobToUser', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->assignJobToUser($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'assignJobToUserResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_assignJobToUser


	/**
	 *
	 * Elimina la relacin usuario-cargo
	 *
	 * @param string $username Nombre de usuario
	 * @param string $password Contrasea del usuario
	 * @param object $jobIdentifier identificador del cargo del que se quiere eliminar la asociacin
	 * @param object $userIdentifier Identificador del usuario del que se quiere eliminar la asociacin
	 * @return	array 	Array asociativo con la respuesta del servicio [xxx]
	 * @access 	public
	 */
	public function pf_adm_separateJobToUser($username, $password, $jobIdentifier, $userIdentifier) {
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_PF_ADM, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_PF_ADM);

			// Nivel de peticion
			$mainWS = $this->pf_doAuth($username, $password, 'authentication', self::$NAMESPACE_PF_ADM);
			$mainWS->append(new SoapVar($jobIdentifier, XSD_STRING, null, null,'jobIdentifier', self::$NAMESPACE_PF_ADM));
			$mainWS->append(new SoapVar($userIdentifier, XSD_STRING, null, null,'userIdentifier', self::$NAMESPACE_PF_ADM));

			// Realiza la peticion
			$miSoapRQ = new SoapVar($mainWS, SOAP_ENC_OBJECT,null,null, 'separateJobToUser', self::$NAMESPACE_PF_ADM);
			$respuesta = $clienteWS->separateJobToUser($miSoapRQ);

			$vRespuesta = array();
			if (is_object($respuesta))
			{
				$vRespuesta = $this->objectTree2array($respuesta);
			}
			return($vRespuesta);
		}
		catch (Exception $e)
		{
			// Comprobamos si estamos ante excepcion MTOM
			$mensaje = $e->getMessage();
			$mensaje = trim(strtolower($mensaje));
			if ($mensaje == 'looks like we got no xml document')
			{
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();

				// Realiza la interpretacion/sustitucion estandar de MTOM
				$vRetorno = $this->tratarMTOM4PF($response, true, "\r\n", 'separateJobToUserResponse');

				return ($vRetorno);
			}
			else
			{
				$this->tratarExcepcionEstandar($e, $clienteWS);
			}
		}
	}//pf_adm_separateJobToUser


	private function tratarMTOM4PF($response, $autoDetectSeparator = false, $crlf = "\n", $itemToReturn = 'respuesta') {
		// Extraer el separador de bloques
		/*
		$mimeBoundary = '--MIME_Boundary';
		if ($autoDetectSeparator === true) {
			$start = 0;	// strpos($response, '<?xml');
			$end = strpos($response, $crlf);
			$mimeBoundary = substr($response, $start, $end-$start);
		}

		// Obtiene la respuesta, y la divide en los bloques de trabajo
		$vBodyResponse = explode($mimeBoundary, $response);	// TODO: SE ASUME LA CADENA SEPARADORA '--MIME_Boundary'
		*/

		// Interpreta e identifica cada bloque
		$oMimeParser = new WSCMimeParser();
		$oMime = $oMimeParser->decode($response);

		// Accede a la respuesta principal
		/*
		$vRespuesta = preg_grep("/<[a-z]*:Envelope.*?>(.*)<\/[a-z]*:Envelope>/i", explode($crlf, $response));
		$vSlice = array_slice($vRespuesta, 0, 1);
		$respuestaSoap = array_shift($vSlice);
		*/

		$vRespuesta = array();
		preg_match("/<[a-z]*:Envelope.*?>(.*)<\/[a-z]*:Envelope>/is", $response, $vRespuesta);
		if (count($vRespuesta)<1)
		{
			throw new Exception('No puede ubicarse RESPONSE dentro de MTOM');
		}
		$respuestaSoap = $vRespuesta[0];

		// Accede a la respuesta
		$dom = new \DOMDocument('1.0');
		$dom->loadXML($respuestaSoap);
		$xpath = new \DOMXpath($dom);
		$vNodoRespuesta = $xpath->query("/*[local-name()='Envelope']/*[local-name()='Body']//*[local-name()='". $itemToReturn . "']");
		$nodoRespuesta = $vNodoRespuesta->item(0);

		// Sustituye los campos MTOM
		$vNodoInclude = $xpath->query("//*[local-name()='Include']");
		foreach ($vNodoInclude as $nodoItem)//Recorremos los nodosXML include y sustituidmos por el contenido
		{
			$subIdBuscado = (string) $nodoItem->getAttribute('href');
			$subIdBuscado = urldecode($subIdBuscado);
			$idBuscado = '<'.substr($subIdBuscado, 4).'>';
			foreach ($oMime->parts as $parte)
			{
				$idAnexoMime = $parte->headers['content-id'];
				if ($idBuscado == $idAnexoMime)
				{
					$contenidoB64 = $parte->body;
					//if ($parte->mimetype =='content/octet-stream')
					if (($parte->mimetype =='content/octet-stream') || ($parte->mimetype =='application/pdf'))
					{
						$contenidoB64 = base64_encode($parte->body);
					}
					$textNode = $dom->createTextNode($contenidoB64);
					$nodoItem->parentNode->replaceChild($textNode, $nodoItem);
				}
			}//Fin for partes
		}//Fin for nodos

		// Convierte la respuesta en un array
		return $this->xml_to_array($nodoRespuesta);
	}//tratarMTOM4PF

}

?>